#include "x86.h"

/*****************************************************************************
 * print111os
 *
 *   This tiny operating system prints "111!" to the console,
 *   then hangs forever!
 *
 *   Some questions:
 *   (1) We're lucky this program is under 510 bytes long.  Why?
 *   (2) Can you fill the rest of the console with stars, instead of spaces?
 *   (3) How would you make "111!" come out in red?
 *
 *****************************************************************************/

void
bootmain(void)
{
	int i;
	// Set the 'console' variable to point at the magic portion of memory
	// that represents the CGA console.
	char *console = (char *) 0xB8000;
	
	// Write '111!' to the console.
	// The CGA console uses 2 bytes for each character, to support
	// extended features like colored characters.
	// The actual character is in the lower byte; the attributes are in
	// the upper byte.
	console[0] = '1';
	console[1] = 0x07;		// This means light gray-on-black text
	console[2] = '1';
	console[3] = 0x07;
	console[4] = '1';
	console[5] = 0x07;
	console[6] = '!';
	console[7] = 0x07;

	// Clear the rest of the console by writing spaces to it.
	for (i = 4; i < 80 * 25; i++) {
		console[i*2] = ' ';
		console[i*2 + 1] = 0x07;
	}

	// These programmed I/O instructions tell the video card to set
	// up a blinking cursor at position 0x0004 -- that is, row 0, column 4,
	// just after the '111!'.
	
	// First, write the upper byte of the cursor position
	outb(0x3D4, 14);		// Command 14 = upper byte of position
	outb(0x3D5, 0x0004 / 256);
	// Second, write the lower byte of the cursor position
	outb(0x3D4, 15);		// Command 15 = lower byte of position
	outb(0x3D5, 0x0004 % 256);
	
	// Now loop forever!
	while (1)
		/* do nothing */;
}
